BUFF_FLASH_TIME = 0.75;
BUFF_ALPHA_VALUE = 1;
BUFF_MAX_ALPHA = 1.2;
BUFF_MIN_ALPHA = 0.3;
BUFF_DURATION_WARNING_TIME = 30;
MAX_BUFF_SIZE = 40;
MAX_DEBUFF_SIZE = 40;
BUFF_LINE_ITEMS = 10;

function PlayerBuffButton_Update(this)
	local name, texture, count;
	if ( this.debuff ) then
		name, texture, count = UnitDebuff("player", this:GetID());
	else
		name, texture, count = UnitBuff("player", this:GetID());
	end

	if ( not name ) then
		this:Hide();
	else
		if ( count < 1 ) then
			count = "";
		end
		getglobal(this:GetName().."Icon"):SetTexture(texture);
		getglobal(this:GetName().."Count"):SetText(count);
		this:SetAlpha(1.0);
		this:Show();
	end
	
	if ( GameTooltip:IsOwned(this) ) then
		if ( this.debuff ) then
			GameTooltip:SetUnitDebuff("player", this:GetID());
		else
			GameTooltip:SetUnitBuff("player", this:GetID());
		end
	end
	
	PlayerBuffButton_OnUpdate(this);
end

function PlayerBuffButton_OnLoad(this)
	this:RegisterForClicks("RightButton");
	this:RegisterEvent("UNIT_BUFF_CHANGED");

	local itemButton;
	for i = 2, MAX_BUFF_SIZE do
		itemButton = getglobal("PlayerBuffButton"..i);
		itemButton:ClearAllAnchors();

		if ( i <= BUFF_LINE_ITEMS ) then
			itemButton:SetAnchor("TOPRIGHT", "TOPLEFT", "PlayerBuffButton"..(i-1), -5, 0);
		else
			itemButton:SetAnchor("TOPRIGHT", "BOTTOMRIGHT", "PlayerBuffButton"..(i-BUFF_LINE_ITEMS), 0, 12);
		end
	end

	for i = 2, MAX_DEBUFF_SIZE do
		itemButton = getglobal("PlayerDebuffButton"..i);
		itemButton:ClearAllAnchors();

		if ( i <= BUFF_LINE_ITEMS ) then
			itemButton:SetAnchor("TOPRIGHT", "TOPLEFT", "PlayerDebuffButton"..(i-1), -5, 0);
		else
			itemButton:SetAnchor("TOPRIGHT", "BOTTOMRIGHT", "PlayerDebuffButton"..(i-BUFF_LINE_ITEMS), 0, 12);
		end
	end

	PlayerBuffButton_Update(this);
end

function PlayerBuffButton_OnEvent(this, event)
	if ( event == "UNIT_BUFF_CHANGED" ) then
		if ( arg1 == "player" ) then
			PlayerBuffButton_Update(this);
		end
	end
end

function PlayerBuffButton_OnUpdate(this, elapsedTime)
	local leftTime;
	if ( this.debuff ) then
		leftTime = UnitDebuffLeftTime("player", this:GetID());
	else
		leftTime = UnitBuffLeftTime("player", this:GetID());
		if ( leftTime and leftTime < BUFF_DURATION_WARNING_TIME ) then
			this:SetAlpha(BUFF_ALPHA_VALUE);
		else
			this:SetAlpha(1.0);
		end
	end
	BuffFrame_UpdateDuration(this, leftTime);
end

function BuffFrame_UpdateDuration(buffButton, leftTime)
	local duration = getglobal(buffButton:GetName().."Duration");
	if ( leftTime ) then
		duration:SetText(SecondsToTimeAbbrev(leftTime));
		if ( leftTime < BUFF_DURATION_WARNING_TIME ) then
			duration:SetColor(HIGHLIGHT_FONT_COLOR.r, HIGHLIGHT_FONT_COLOR.g, HIGHLIGHT_FONT_COLOR.b);
		else
			duration:SetColor(NORMAL_FONT_COLOR.r, NORMAL_FONT_COLOR.g, NORMAL_FONT_COLOR.b);
		end
		duration:Show();
	else
		duration:Hide();
	end
end

function PlayerBuffFrame_OnLoad(this)
	BUFF_ALPHA_VALUE = 1;
	BUFF_FLASH_STATE = 1;
end

function PlayerBuffFrame_OnUpdate(this, elapsedTime)
	if ( BUFF_FLASH_STATE ) then
		BUFF_ALPHA_VALUE = BUFF_ALPHA_VALUE - elapsedTime * BUFF_FLASH_TIME;
		if ( BUFF_ALPHA_VALUE < BUFF_MIN_ALPHA ) then
			BUFF_ALPHA_VALUE = BUFF_MIN_ALPHA;
			BUFF_FLASH_STATE = nil;
		end
	else
		BUFF_ALPHA_VALUE = BUFF_ALPHA_VALUE + elapsedTime * BUFF_FLASH_TIME;
		if ( BUFF_ALPHA_VALUE > BUFF_MAX_ALPHA ) then
			BUFF_ALPHA_VALUE = BUFF_MAX_ALPHA;
			BUFF_FLASH_STATE = 1;
		end
	end
end